<?php
namespace Evoluted\EvDiscount\DiscountRestrictions;

use Evoluted\PriceModifier\DiscountRestrictions\Basic;

/**
 * The basic discount restriction handler allows all discount types through without
 * restricting them.
 *
 * @package 	PriceModifier
 * @author 		Sam Biggins <sam@evoluted.net>
 * @author		Evoluted New Media <developers@evoluted.net>
 * @license     http://mit-license.org/
 *
 * @link		https://github.com/evolutednewmedia/pricemodifier
 *
 */
class DiscountRestrictions extends Basic {

	/**
	 * Sets the restrictions on the basket.
	 *
	 * @return void
	 */
	public function setRestrictions() {
		// Setup discounts array incase there are no discount restrictions
		$this->basket->validDiscounts = [];

		foreach ($this->basket->discountRules as $rule) {
			$ruleEnabled = true;

			if ($rule['min_order'] > 0) {
				if ($rule['min_order'] > $this->basket->getBasketTotal()) {
					$ruleEnabled = false;
				}
			}

			if (isset($rule['code_restrictions']) && ! empty($rule['code_restrictions'])) {

				// Go through each of the basket items and see if they rule applies to them.
				foreach ($this->basket->items() as $basketItemId => $basketItem) {
					// This code has restrictions that need checking
					$checkDiscountRestriction = new \CakeEvent('EvBasket.Component.BasketItem.CheckDiscountRestrictions', $basketItem, array(
						'discount' => $rule,
						'basket_item' => array_merge(['id' => $basketItemId], $basketItem->toArray()),
					));
					\CakeEventManager::instance()->dispatch($checkDiscountRestriction);
					// If false is returned discount isn't available for this item
					$results = $checkDiscountRestriction->result;

					// If no items are valid it won't add the discount rule to the list of valid discounts
					if ($results['discountValue'] == true) {
						$this->basket->validDiscounts[$rule['id']][] = $basketItemId;
					}
				}

				// Check for delivery discount restrictions
				$checkDeliveryDiscountRestriction = new \CakeEvent('EvBasket.Component.BasketDelivery.CheckDiscountRestrictions', $this->basket, array(
						'discount' => $rule,
						'basket' => $this->basket->toArray(),
				));
				\CakeEventManager::instance()->dispatch($checkDeliveryDiscountRestriction);
				// If false is returned discount isn't available for the selected delivery method on this basket
				$results = $checkDeliveryDiscountRestriction->result;

				if ($results['discountValue'] == true && $ruleEnabled) {
					$this->basket->validDiscounts[$rule['id']] = true;
				}

			} else {
				// Not restricted by item so add discount code with no product restrictions
				if ($ruleEnabled) {
					$this->basket->validDiscounts[$rule['id']] = true;
				}
			}
		}
	}
}
